#ifndef DACCONFIG_KEYS_H
#define DACCONFIG_KEYS_H

#define DAC_IO_SET_CONFIG_10V_FULL_SCALE  0x0001
#define DAC_IO_SET_CONFIG_20V_FULL_SCALE  0x0002
#define DAC_IO_SET_CONFIG_SCALING         0x0003
#define DAC_IO_SET_CONFIG_OFFSET          0x0004
#define DAC_IO_SET_CONFIG_SCALING_CAL     0x0005
#define DAC_IO_SET_CONFIG_OFFSET_CAL      0x0006

#endif // DACCONFIG_KEYS_H
