#ifndef ADCIO_H
#define ADCIO_H

#ifdef __cplusplus
extern "C" {
#endif

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <pkgconf/hal.h>
#include <cyg/infra/cyg_type.h>
#include <cyg/io/adcconfig_keys.h>

//----------------------
// Default configuration
//----------------------

#ifdef __cplusplus
}
#endif

#endif // ADCIO_H
