#ifndef DACIO_H
#define DACIO_H

#ifdef __cplusplus
extern "C" {
#endif

//-----------------------------------
// Includes and forward deeclarations
//-----------------------------------
#include <pkgconf/hal.h>
#include <cyg/infra/cyg_type.h>
#include <cyg/io/dacconfig_keys.h>

//----------------------
// Default configuration
//----------------------

#ifdef __cplusplus
}
#endif

#endif // DACIO_H
