#ifndef ENCODERCONFIG_KEYS_H
#define ENCODERCONFIG_KEYS_H

#define ENCODER_IO_SET_CONFIG_LATCHMODE            0x0001
#define ENCODER_IO_SET_CONFIG_LATCHENABLE          0x0002
#define ENCODER_IO_SET_CONFIG_LATCHDISABLE         0x0003
#define ENCODER_IO_SET_CONFIG_OFFSET               0x0004

#define ENCODER_IO_GET_CONFIG_LATCHED              0x0001
#define ENCODER_IO_GET_CONFIG_LATCHPOSITION        0x0002

#endif // ENCODERCONFIG_KEYS_H
